*! version 1.1.1  01may1997
program define areg
	version 5.0
	local options "Level(integer $S_level)"
	if substr("`1'",1,1)=="," | "`*'"=="" {
		if "$S_E_cmd"!="areg" {
			error 301
		}
		parse "`*'"
		Di_areg  `level'  /* display results */
		exit
	}

	local varlist "req ex"
	local if "opt"
	local in "opt"
	local weight "aweight fweight pweight"
	local options "`options' Absorb(string) Robust CLuster(string)"
	parse "`*'"
	parse "`varlist'", parse(" ")
	local y `1'
	macro shift

	if "`absorb'"=="" {
		di in red "absorb() required"
		exit 100 
	}
	unabbrev `absorb'
	local absorb "$S_1"
	local n : word count `absorb'
	if `n' > 1 {
		di in red "only one variable allowed in absorb() option"
		exit 103 
	}
	if "`cluster'"!="" {
		unabbrev `cluster'
		local cluster "$S_1"
		local n : word count `cluster'
		if `n' > 1 {
			di in red "only one variable allowed " /*
			*/ "in cluster() option"
			exit 103 
		}
		local clopt  "cluster(`cluster')"
		local robust "robust"
	}
	if "`weight'"=="pweight" {
		local weight "aweight"
		local robust "robust"
	}
	if "`robust'"!="" {
		local mse1 "mse1"
	}

	tempvar touse x c
	tempname vadj b V
	quietly {

	/* Do mark/markout. */

		mark `touse' [`weight'`exp'] `if' `in'
		markout `touse' `varlist'
		markout `touse' `absorb' `cluster', strok
		count if `touse'
		if      _result(1) == 0 { noisily error 2000 }
		else if _result(1) == 1 { noisily error 2001 }

	/* Preserve and keep. */

		preserve
		keep if `touse'

		if "`weight'"!="" {
			tempvar w
			gen double `w' `exp'
			local exp "=`w'"
		}

		keep `varlist' `absorb' `cluster' `w'

		if "`weight'"=="" { local w 1 }

	/* Sort and count number of groups. */

		sort `absorb'
		count if `absorb'!=`absorb'[_n-1]
		local dfa = _result(1) - 1

	/* If not robust, get R^2 from regression without fixed-effects.
	   This is used for test of fixed-effects.
	*/
		if "`robust'"=="" {
			regress `varlist' [`weight'`exp']
			local r2c = _result(7)
			local dfe = _result(5) - `dfa'
			if `dfe'<=0 | `dfe'==. { noisily error 2001 }
				/* Note: This check is done later, too. */
		}

	/* Remove means from varlist. */

		summ `y' [`weight'`exp']
		local sst = (_result(1)-1)*_result(4)

		by `absorb': gen double `x' = sum(`w'*`y')/sum(`w')
		by `absorb': replace `x' = `y'-`x'[_N]+_result(3)
		drop `y'
		rename `x' `y'

		local i 1
		while "``i''"!="" {
			summ ``i'' [`weight'`exp']
			by `absorb': gen double `x' = sum(`w'*``i'')/sum(`w')
			by `absorb': replace `x' = ``i''-`x'[_N]+_result(3)
			drop ``i''
			gen float ``i'' = `x'
			drop `x'
			local i = `i' + 1
		}

	/* Do regression. */

		regress `varlist' [`weight'`exp'], `mse1'

		local nobs = _result(1)
		local nvar = _result(3)		/* # ind. variables in model */
		local mdf  = _result(3) + `dfa'	/* true model df */
		local sse  = _result(4)
		local r2   = 1 - `sse'/`sst'
		local dfe  = `nobs' - 1 - `mdf'
		if `dfe'<=0 | `dfe'==. { noisily error 2001 }

	/* Adjust covariance matrix for correct dfe and post. */

		matrix `b' = get(_b)
		matrix `V' = get(VCE)

		if "`robust'"=="" {
			scalar `vadj' = _result(5)/`dfe'
			matrix `V' = `vadj'*`V'
		}
		else { /* compute robust variance */

			local k = `mdf' + 1
			tempvar res
			predict double `res', res

			_robust `res' [`weight'`exp'], v(`V') minus(`k') /*
			*/	`clopt'

			local cn = _result(3)
		}

		matrix post `b' `V', dof(`dfe') obs(`nobs') depn(`y')

		if "`*'"!="" {
			test `*', min

			global S_E_mdf = _result(3)
			global S_E_f   = _result(6)
		}
		else {
			global S_E_mdf 0
			global S_E_f   .
		}

		if "`robust'"=="" {
			global S_E_f2 = ((`r2'-`r2c')/`dfa')/((1-`r2')/`dfe')
		}
		else {
			global S_E_vce "Robust"
			global S_E_cn  "`cn'"
			global S_E_cvn "`cluster'"
		}
		global S_E_nobs "`nobs'"
		global S_E_sse  "`sse'"
		global S_E_sst	"`sst'"
		global S_E_r2  = 1 - `sse'/`sst'
		global S_E_tdf  "`dfe'"
		global S_E_abs	"`absorb'"
		global S_E_dfa  "`dfa'"
		global S_E_depv "`y'"
		global S_E_cmd  "areg"
	}

	/* Display results. */

	Di_areg  `level'
end

program define Di_areg
	version 5.0
	local level "`1'"

	local rmse = sqrt($S_E_sse/$S_E_tdf)
	local p   = fprob($S_E_mdf,$S_E_tdf,$S_E_f)
	local ar2 = 1 - (1-$S_E_r2)*($S_E_nobs - 1)/$S_E_tdf

	if "$S_E_vce"=="Robust" {
		local title "Regression with robust standard errors"
	}

	di _n in gr "`title'" _col(56) /*
	*/	"Number of obs =" in ye %8.0f $S_E_nobs

	di in gr _col(56) "F(" in ye %3.0f $S_E_mdf in gr "," in ye  /*
	*/	%6.0f $S_E_tdf in gr ") =" in ye %8.2f $S_E_f
	di in gr _col(56) "Prob > F      ="  in ye %8.4f `p'
	di in gr _col(56) "R-squared     ="  in ye %8.4f $S_E_r2
	di in gr _col(56) "Adj R-squared ="  in ye %8.4f `ar2'
	di in gr _col(56) "Root MSE      = " in ye %7.0g `rmse' _n

	matrix mlout, level(`level')

	if "$S_E_vce"=="Robust" { FooterR }
	else			  Footer
end

program define Footer
	version 5.0
	local skip  = 8 - length("$S_E_abs")
	local dfa1  = $S_E_dfa + 1
	local skip2 = max(14-length("`dfa1'"),0)
	local todisp "F($S_E_dfa,$S_E_tdf) = "
	local skip3 = max(23-length("`todisp'"),0)

	di _skip(`skip') in gr "$S_E_abs |" _skip(`skip3') "`todisp'" /*
	*/ in ye %10.3f $S_E_f2 %8.3f fprob($S_E_dfa,$S_E_tdf,$S_E_f2) /* 
	*/ in gr _skip(`skip2') "(`dfa1' categories)"
end

program define FooterR
	version 5.0
	local skip  = 8 - length("$S_E_abs")
	local dfa1  = $S_E_dfa + 1
        local skip2 = max(44 - length("`dfa1'"), 0)

        di _skip(`skip') in gr "$S_E_abs |   absorbed" /*
        */ _skip(`skip2') "(`dfa1' categories)"
end
