*! version 5.0.3  13oct1997  (also see ereg_lf.ado) 
program define ereg
	version 5.0

	if "`1'"=="" | substr("`1'",1,1)=="," {
		if "$S_E_cmd"!="ereg" { error 301 }
		if "$S_E_frm2"=="hazard" {
			local options "HR"
		}
		else	local options "TR"
		local options "`options' noCOEF noHEAD Level(integer $S_level)"
		parse "`*'"
	}
	else {
		local varlist "req ex min(1)"
		local if "opt"
		local in "opt"
		local weight "fweight aweight iweight pweight noprefix"
		local options "CLuster(string) noCOEF HAzard noHEAD Level(integer $S_level) noLOG Dead(string) HR T0(string) TR Robust SCore(string) *"
		parse "`*'"
		parse "`varlist'", parse(" ")
		local t "`1'"
		mac shift 1 
		local rhs "`*'"

		if "`hazard'"!="" {
			if "`tr'"!="" {
				di in red "tr invalid with hazard option"
				exit 198
			}
		}
		else if "`hr'"!="" {
			local hazard "hazard"
		}

		if "`t0'"!="" {
			unabbrev `t0', max(1) 
			local t0 "$S_1"
		}

		if "`weight'"!="" {
			tempvar wv
			qui gen double `wv' = `exp'
			local w "[`weight'=`wv']"
			if "`weight'"=="pweight" { 
				local robust "robust"
			}
		}
		if "`score'"!="" {
			local n : word count `score'
			if `n'!=1 { 
				di in red "score() invalid"
				exit 198 
			}
			confirm new var `score'
		}
		if "`cluster'"!="" {
			unabbrev `cluster', max(1)
			local cluster "$S_1"
			local robust "robust"
		}
		if "`robust'"!="" & "`score'"=="" {
			tempvar score
		}

		tempvar touse 
		mark `touse' `w' `if' `in'
		markout `touse' `t' `rhs' `dead' `t0'
		markout `touse' `cluster', strok


		if "`dead'"!="" {
			unabbrev `dead', max(1)
			local dead "$S_1"
			capture assert `dead'==0 | `dead'==1 if `touse'
			if _rc {
				tempvar mydead
				qui gen byte `mydead'=`dead'!=0 if `touse'
				local dead `mydead'
			}
		}
		else {
			tempvar dead
			qui gen byte `dead'=1
		}

		if "`t0'"=="" {
			local t0 0
			local realt "`t'"
		}
		else {
			tempvar realt
			qui gen double `realt' = `t'-`t0' if `touse'
			qui compress `realt'
		}
		capture assert `t0'<`t' if `touse'
		if _rc {
			di in red "`t0' >= `t' in some obs."
			exit 498
		}

		_rmcoll `rhs' `w' if `touse'
		local rhs "$S_1"
		global S_1

		qui count if `touse' 
		local nobs = _result(1)		/* may be reset subsequently */
		if `nobs'<2 {
			if _result(1)==0 { error 2000 } 
			error 2001
		}

		if "`weight'" != "" {
			qui replace `wv'=0 if !`touse'
			if "`weight'"=="aweight" {
				qui summ `wv' if `touse', meanonly
				qui replace `wv' = `wv'/_result(3) if `touse'
			}
			else if "`weight'"=="fweight" {
				qui summ `wv', meanonly
				local nobs = round(_result(18),1)
			}
			global EREGw "`wv'"
		}
		else	global EREGw "`touse'"

		global EREGd "`dead'"
		global EREGt "`realt'"

		tempvar mysamp num den
		tempname b f V  b0

		quietly {
			replace $EREGw = 0 if $EREGw==.
			gen double `num' = sum($EREGw*$EREGd)
			gen double `den' = sum($EREGw*$EREGt)
			local cons = ln(`num'[_N]/`den'[_N])
			drop `num' `den'
			gen double `num' = sum($EREGw*ln($EREGd*$EREGt))
			global EREGa = `num'[_N]
			drop `num'

			matrix `b0' = ( `cons' )
			matrix colnames `b0' = _cons
			ereg_lf `b0' `f' - - , fast(0)
			local f0 = scalar(`f')
			scalar drop `f'
		}

		ml begin 
		ml function ereg_lf
		ml method deriv2 
		eq `t' : `t' `rhs'
		ml model `b' = `t', from(`b0')
		ml sample `mysamp', noauto	/* noauto for speed */
		if "`log'"=="" {
			ml maximize `f' `V', `options'
		}
		else	qui ml maximize `f' `V', `options'

		if "`score'"!="" {
			Getscore `b' -> `score'	/* in the hazard metric */
			if "`hazard'"=="" {
				qui replace `score' = -`score'
			}
		}

		if "`hazard'"=="" {
			tempname zero
			local nc = colsof(`b')
			mat `zero' = J(1,`nc',0)
			mat `b' = `zero'-`b'
			mat drop `zero'
			local ttl "log expected time form"
		}
		else {
			local ttl "log relative hazard form"
		}

		ml post ereg, lf0(`f0') /* pr2 */ obs(`nobs') title("`ttl'")
		global S_E_cmd
		global S_E_t0 "`t0'"
		eq drop `t'
		if "`robust'"!="" {
			if "`cluster'"!="" {
				local cluster "cluster(`cluster')"
			}
			_robust `score' `w' if `touse', `cluster' nofatal
			capture test `rhs', min
			global S_E_chi2 = _result(6)
			global S_E_mdf = _result(3)
		}
		if "`hazard'"=="" {
			global S_E_frm2 "time"
		}
		else {
			global S_E_frm2 "hazard"
		}
		global S_E_cmd ereg
		global EREGw
		global EREGd
		global EREGt
		global EREGa
	}

	if "`coef'"=="" {
		if "`hr'" != "" {
			local hr "eform(Haz. Ratio)"
		}
		else if "`tr'" !="" {
			local hr "eform(Tm. Ratio)"
		}
		if "`head'"=="" {
			di _n in gr /*
			*/ "Exponential regression -- entry time $S_E_t0" _c
			ml mlout ereg, `hr' level(`level')
		}
		else	mat mlout, `hr' level(`level')
	}
end

program define Getscore /* b -> score */
	local b "`1'"
	local sco "`3'"
	tempvar I
	quietly {
		matrix score double `I' = `b' if $EREGw
		gen double `sco' = $EREGd-exp(`I')*$EREGt if $EREGw
	}
end
exit

globals used:

	macro EREGd	name of 0/1 dead variable
	macro EREGt	name of time variable
	macro EREGw	weight variable; 0 if obs. not used
			fweight:  0 or integer
			aweight:  0 or pos. real, sums to _N
			iweight:  0, or real,

	macro EREGa	(#) adjustment to log likelihood function
