*! version 2.2.5   20jul1998 
program define kdensity
	version 5.0

	local varlist "req ex min(1) max(1)"
	local if      "opt"
	local in      "opt"
	local weight  "fweight aweight"
	#delimit ;
	local options  "Generate(string) N(integer 50) Width(real 0.0) 
			noGRaph BIweight COSine EPan GAUss RECtangle PARzen
			TRIangle Symbol(string) Connect(string) Title(string)
			noDENsity AT(string) NORmal STUd(int 0) *" ;
	#delimit cr

	parse "`*'"
	parse "`varlist'", parse(" ,")
	if "`2'" != "" {
		di in red "only one variable allowed in varlist"
		exit
	}
	local ix "`1'"
	local ixl: variable label `1'
	if "`ixl'"=="" { 
		local ixl "`1'"
	}

	if "`density'" == "nodensity" {		/* Fix sense of option */ 
		local density ""
	}
	else {
		local density "density"
	}

	local gen "`generat'"

	local kflag = ( ("`epan'" != "") + ("`biweigh'" != "") + /*
			*/ ("`triangl'" != "") + ("`gauss'" != "") + /*
			*/ ("`rectang'" != "")  + ("`parzen'" != ""))
	if `kflag' > 1 {
		di in red "only one kernel may be specified"
		exit 198
	}

	if "`biweigh'"       != "" { local kernel="Biweight"     }
	else if "`cosine'"   != "" { local kernel="Cosine"       }
	else if "`triangl'"  != "" { local kernel="Triangle"     }
	else if "`gauss'"    != "" { local kernel="Gaussian"     }
	else if "`rectang'"  != "" { local kernel="Rectangular"  }
	else if "`parzen'"   != "" { local kernel="Parzen"       }
	else                       { local kernel="Epanechnikov" }

	tempvar use

	quietly {
		mark `use' [`weight'`exp'] `if' `in'
		markout `use' `varlist' 
		count if `use'
		local nnobs = _result(1)
	}			
	if _result(1)==0 { error 2000 } 

	parse "`gen'", parse(" ")
	local wc : word count `*'
	if `wc' { 
		if `wc' == 1 {
			if "`at'" == "" {
				error 198
			}
			confirm new var `1'
			local yl  "`1'"
			local xl "`at'"
			local nsave 1
		}
		else {
			if `wc' != 2 { error 198 }
			confirm new var `1'
			confirm new var `2'
			local xl  "`1'"
			local yl  "`2'"
			local nsave 2		
		}
	}
	else {
		if "`graph'"!="" {
			di in bl /*
*/ "no graph requested and did not request results be saved; no action taken"
			exit
		}
		local xl   "X"
		local yl   "Density"	
		local nsave 0		
	}
		
	tempvar d m z y
	qui gen double `d'=.
	qui gen double `y'=.
	qui gen double `z'=.
	qui gen double `m'=.

	if "`at'" != "" {		/* Sort because of missing vals */
		confirm var `at'
		qui count if `at' != . 
		local n = _result(1)
		local srtlist : sortedby
		tempvar obsvar
		gen `obsvar' = _n
		qui replace `m' = `at' 
		sort `m' `obsvar'
	}
	else {

		if "`n'"!= "" {
			if `n' <= 1 { local n = 50 }
			if `n' > _N { 
				local n = _N
				noi di in gr "(n() set to " `nnobs' ")"
			}
		}
	}

        if "`weight'" != "" {
                tempvar tt
                qui gen double `tt' `exp' if `use'
                qui summ `tt' 
		if "`weight'" == "aweight" {
			qui replace `tt' = `tt'/_result(3)
		}
        }
        else {
                local tt = 1
        }

	quietly summ `ix' [`weight'`exp'] if `use', detail
	local nmean = _result(3)
	local nsig = _result(4)

	tempname wwidth
	scalar `wwidth' = `width'
	if `wwidth' <= 0.0 { 
		scalar `wwidth' = min( sqrt(_result(4)), /*
				*/ (_result(11)-_result(9))/1.349)
		scalar `wwidth' = 0.9*`wwidth'/(_result(1)^.20)
	}

	tempname delta wid
	scalar `delta' = (_result(6)-_result(5)+2*`wwidth')/(`n'-1)
	scalar `wid'   = _result(1) * `wwidth'

	if "`at'" == "" {
		qui replace `m' = _result(5)-`wwidth'+(_n-1)*`delta' in 1/`n'
	}

	local i 1
	if "`biweigh'" != "" {
		local con1 = .9375
		while `i'<=`n' {
			qui replace `z'=(`ix'-`m'[`i'])/(`wwidth') /*
				*/ if `use' & `use'
			qui replace `y'=`tt'*`con1'*(1-(`z')^2)^2 /*
				*/ if abs(round(`z',1e-8))<1
			qui summ `y' if `y'!=.
			qui replace `d'=(_result(3)*_result(1))/`wid' in `i'
			qui replace `y'=.
			local i = `i'+1
		}
		qui replace `d'=0 if `d'==. in 1/`n'
	}
	else if "`cosine'" != "" {
		while `i'<=`n' {
			qui replace `z'=(`ix'-`m'[`i'])/(`wwidth') /*
				*/ if `use' & `use'
			qui replace `y'= `tt'*(1+cos(2*_pi*`z')) /*
				*/ if abs(round(`z',1e-8))<0.5
			qui summ `y' if `y'!=.
			qui replace `d'=(_result(3)*_result(1))/`wid' in `i'
			qui replace `y'=.
			local i = `i'+1
		}
		qui replace `d'=0 if `d'==. in 1/`n'
	}
	else if "`triangl'" != "" {
		while `i'<=`n' {
			qui replace `z'=(`ix'-`m'[`i'])/(`wwidth') if `use'
			qui replace `y'= `tt'*(1-abs(`z')) /*
				*/ if abs(round(`z',1e-8))<1
			qui summ `y' if `y'!=.
			qui replace `d'=(_result(3)*_result(1))/`wid' in `i'
			qui replace `y'=.
			local i = `i'+1
		}
		qui replace `d'=0 if `d'==. in 1/`n'
	}
	else if "`parzen'" != "" {
		local con1 = 4/3
		local con2 = 2*`con1'
		while `i'<=`n' {
			qui replace `z'=(`ix'-`m'[`i'])/(`wwidth') if `use'
			qui replace `y'= `con1'-8*(`z')^2+8*abs(`z')^3 /*
				*/ if abs(round(`z',1e-8))<=.5
			qui replace `y'= `tt'*`con2'*(1-abs(`z'))^3       /*
				*/ if abs(round(`z',1e-8))>.5 & /*
				*/ abs(round(`z',1e-8))<1
			
			qui summ `y' if `y'!=.
			qui replace `d'=(_result(3)*_result(1))/`wid' in `i'
			qui replace `y'=.
			local i = `i'+1
		}
		qui replace `d'=0 if `d'==. in 1/`n'
	}
	else if "`gauss'" != "" {
		local con1 = sqrt(2*_pi)
		while `i'<=`n' {
			qui replace `z'=(`ix'-`m'[`i'])/(`wwidth') if `use'
			qui replace `y'= `tt'*exp(-0.5*((`z')^2))/`con1'
			qui summ `y'  
			qui replace `d'=(_result(3)*_result(1))/`wid' in `i'
			local i = `i'+1
		}
		qui replace `d'=0 if `d'==. in 1/`n'
	}
	else if "`rectang'" != "" {
		while `i'<=`n' {
			qui replace `z'=(`ix'-`m'[`i'])/(`wwidth') if `use'
			qui replace `y'= `tt'*0.5 if abs(round(`z',1e-8))<1
			qui summ `y' if `y'!=.
			qui replace `d'=(_result(3)*_result(1))/`wid' in `i'
			qui replace `y'=.
			local i = `i'+1
		}
		qui replace `d'=0 if `d'==. in 1/`n'
	}
	else {
		local con1 = 3/(4*sqrt(5))
		local con2 = sqrt(5)
		while `i'<=`n' {
			qui replace `z'=(`ix'-`m'[`i'])/(`wwidth') if `use'
			qui replace `y'= `tt'*`con1'*(1-((`z')^2/5)) /* 
				*/  if abs(round(`z',1e-8))<=`con2'
			qui summ `y' if `y'!=.
			qui replace `d'=(_result(3)*_result(1))/`wid' in `i'
			qui replace `y'=.
			local i = `i'+1
		}
		qui replace `d'=0 if `d'==. in 1/`n'
	}

	label var `d' "`yl'"
	label var `m' "`ixl'"

	qui summ `d' in 1/`n'
	local scale = 1/(`n'*_result(3))

	if "`density'" == "" {
		qui replace `d' = `d'*`scale' in 1/`n'
	}

	if "`graph'"=="" {
		if "`symbol'"  == "" {local symbol "o"}
		if "`connect'" == "" {local connect "l" }
		if "`title'"   == "" {
			local title "Kernel Density Estimate"
		}
		if "`normal'" != "" {
			tempvar znorm 
			qui gen `znorm' = 1/sqrt(2*_pi*`nsig')* /*
				*/ exp(-0.5*(`m'-`nmean')^2/`nsig')
			local symbol "`symbol'i"
			local connect "`connect'l"
			if "`density'" == "" {
				tempvar fz
				qui gen `fz' = sum(`znorm')
				qui replace `znorm' = `znorm'/`fz'[_N]
			}
		}
		if `stud' > 0 {
			tempvar tm
			qui gen `tm' = exp(lngamma((`stud'+1)/2)) /* 
				*/ / exp(lngamma(`stud'/2)) /*
				*/ * 1/sqrt(`stud'*_pi) * /*
				*/ 1/((1+((`m'-`nmean') /*
				*/ / sqrt(`nsig') )^2 /* 
				*/ /`stud')^((`stud'+1)/2))
			local symbol "`symbol'i"
			local connect "`connect'l"
			tempvar ft
			qui gen `ft' = sum(`tm')
			if "`density'" == "" {
				qui replace `tm' = `tm'/`ft'[_N]
			}
			else {
				qui replace `tm' = `tm'/`ft'[_N]/`scale'
			}
		}
		graph `d' `znorm' `tm' `m', s(`symbol') c(`connect') /*
		*/ title(`title') `options'
	}
	global S_1   "`kernel'"
	global S_3 = `wwidth'
	global S_2 = `n'
	global S_4 = `scale'

	if `nsave' == 1 {
		label var `d' "density: `ixl'"
		rename `d' `yl'
	}
	else if `nsave' == 2 {
		label var `m' "`ixl'"
		label var `d' "density: `ixl'"
		rename `d' `yl'
		rename `m' `xl'
	}
	if "`srtlist'" != "" {
		sort `srtlist' `obsvar'
	}
end
