*! version 4.0.1  31oct1997
program define ttest
	version 5.0
	local varlist "req ex max(1)"
	local exp "opt noprefix"
	local if "opt"
	local in "opt"
	local options "BY(string) UNPaired UNEqual Welch Level(int $S_level)"
	parse "`*'"
	tempvar touse
	mark `touse' `if' `in'

	if "`exp'"!="" {
		if "`by'"!="" {
			di in red "may not combine = and by()"
			exit 198
		}

		if "`unpaire'"!="" { /* do two-sample (unpaired) test */

			confirm variable `exp'

			_ttest two ttesti `level' `touse' `varlist' `exp' /*
			*/ `unequal' `welch'
			exit
		}

		/* If here, we do one-sample (paired) test. */

		if "`unequal'"!="" {
			di in red "unequal option invalid for one-sample test"
			exit 198
		}
		if "`welch'"!="" {
			di in red "welch option invalid for one-sample test"
			exit 198
		}

		capture confirm number `exp'
		if _rc==0 {
			_ttest one ttesti `level' `touse' `varlist' `exp'
			exit
		}

		confirm variable `exp'

		Paired `level' `touse' `varlist' `exp'

		exit
	}

	/* If here, do two-sample (unpaired) test with by(). */

	if "`by'"=="" {
                di in red "by() option required"
                exit 100
        }

        confirm variable `by'

        local nbyvars : word count `by'
        if `nbyvars' > 1 {
                di in red "only variable allowed in by()"
                exit 103
        }

        _ttest by ttesti `level' `touse' `varlist' `by' `unequal' `welch'
end

program define Paired
	local level "`1'"
	local touse "`2'"
	local xvar1 "`3'"
	local xvar2 "`4'"

        global S_1  /* will contain #obs 1 or #obs               */
        global S_2  /*              mean 1 or mean               */
        global S_3  /*              #obs 2 (or empty)            */
        global S_4  /*              mean 2 (or empty)            */
        global S_5  /*              df                           */
        global S_6  /*              t                            */
        global S_7  /*              two-sided p-value            */
        global S_8  /*              left  one-sided p-value      */
        global S_9  /*              right one-sided p-value      */
        global S_10 /*              std. err. (denominator of t) */
        global S_11 /*              std. dev. 1                  */
        global S_12 /*              std. dev. 2                  */
        global S_13 /*              empty for paired             */

	unabbrev `xvar2'
	local xvar2 "$S_1"

	tempvar diff

	quietly {
		gen double `diff' = `xvar1' - `xvar2' if `touse'

		summarize `diff'
		local n = _result(1)
		local m = _result(3)
		local s = sqrt(_result(4))

		if `n' == 0 { noisily error 2000 }

		summarize `xvar1' if `diff'!=.
		local m1 = _result(3)
		local s1 = sqrt(_result(4))

		summarize `xvar2' if `diff'!=.
		local m2 = _result(3)
		local s2 = sqrt(_result(4))
	}

/* Compute statistics. */

        local t  = `m'*sqrt(`n')/`s'
        local se = `s'/sqrt(`n')

        local p = tprob(`n'-1,`t')
        if `t' < 0 {
                local pl = `p'/2
                local pr = 1 - `pl'
        }
        else {
                local pr = `p'/2
                local pl = 1 - `pr'
        }

/* Display table of mean, std err, etc. */

	di _n in gr "Paired t test"

	_ttest header `level' `xvar1'

	_ttest table `level' `xvar1' `n' `m1' `s1'
	_ttest table `level' `xvar2' `n' `m2' `s2'
	di in gr "---------+" _dup(68) "-"

	_ttest table `level' "diff" `n' `m' `s'

	di in gr _dup(78) "-" _n

/* Display Ho. */

	_ttest center "Ho: mean(@`xvar1'@ - @`xvar2'@) = mean(diff) = 0"

/* Display Ha. */

        local tt : di %8.4f `t'
        local p1 : di %8.4f `pl'
        local p2 : di %8.4f `p'
        local p3 : di %8.4f `pr'

        di
        _ttest center "Ha: mean(diff) < 0"  /*
	*/            "Ha: mean(diff) ~= 0" /*
	*/            "Ha: mean(diff) > 0"

        _ttest center "    t = @`tt'@"    /*
        */            "      t = @`tt'@"  /*
        */            "    t = @`tt'@"
        _ttest center "P < t = @`p1'@"   /*
        */            "P > |t| = @`p2'@" /*
        */            "P > t = @`p3'@"

/* Save results. */

        global S_1  `n'
        global S_2  `m1'
	global S_3  `n'
	global S_4  `m2'
        global S_5 =`n' - 1
	global S_6  `t'
	global S_7  `p'
	global S_8  `pl'
	global S_9  `pr'
	global S_10 `se'
        global S_11 `s1'
        global S_12 `s2'
end
