*! version 3.0.0  31oct1997
program define ttesti
	version 5.0

	global S_1  /* will contain #obs 1 or #obs                */
   	global S_2  /*              mean 1 or mean                */
	global S_3  /*              #obs 2 (or empty)             */
	global S_4  /*              mean 2 (or empty)             */
	global S_5  /*              df                            */
	global S_6  /*              t                             */
        global S_7  /*              two-sided p-value             */
        global S_8  /*              left  one-sided p-value       */
        global S_9  /*              right one-sided p-value       */
	global S_10 /*              std. err. (denominator of t)  */
	global S_11 /*              std. dev. 1 or std. dev.      */
	global S_12 /*              std. dev. 2 (or empty)        */
	global S_13 /*              combined std. dev. (or empty) */

/* Parse. */

	parse "`*'", parse(" ,")

	if "`5'"=="" | "`5'"=="," { /* Do one-sample test */

		local args "`1' `2' `3' `4'"
                macro shift 4
                local options "Xname(string) Yname(string) Level(int $S_level)"
                parse "`*'"
                if `level' < 10 | `level' > 99 {
                        local level 95
                }

                _ttest check ttest one `args' /* check numbers */

                OneTest `args' `level' "`xname'"

                exit
        }

/* Here only if two-sample test. */

        local arg1 "`1' `2' `3'"
        local arg2 "`4' `5' `6'"
        macro shift 6
        local options /*
	*/ "Xname(string) Yname(string) UNEqual Welch Level(int $S_level)"
        parse "`*'"
        if `level' < 10 | `level' > 99 {
                local level 95
        }
	if "`welch'"!="" & "`unequal'"=="" {
		di in red "welch invalid without unequal specified"
		exit 198
	}

        _ttest check ttest first  `arg1' /* check numbers */
        _ttest check ttest second `arg2' /* check numbers */

        TwoTest `arg1' `arg2' `level' "`xname'" "`yname'" "`unequal'" "`welch'"
end

program define OneTest
	local n     "`1'"
        local m     "`2'"
        local s     "`3'"
        local m0    "`4'"
        local level "`5'"
        local xname "`6'"

/* Compute statistics. */

	if `n' == 1 { local s . } /* error check allows std dev to be 0 */

	local t = (`m' - `m0')*sqrt(`n')/`s'
	local se = `s'/sqrt(`n')

	local p = tprob(`n'-1,`t')
	if `t' < 0 {
		local pl = `p'/2
		local pr = 1 - `pl'
	}
	else {
		local pr = `p'/2
		local pl = 1 - `pr'
	}

/* Display table of mean, std err, etc. */

	di _n in gr "One-sample t test"

	_ttest header `level' "`xname'"

	if "`xname'"=="" { local xname "x" }

	_ttest table `level' "`xname'" `n' `m' `s'
	di in gr _dup(78) "-" _n "Degrees of freedom: " in ye `n'-1 _n

/* Display Ho. */

	if length("`m0'") > 8 {
		local m0 : di %9.0g `m0'
		local m0 = trim("`m0'")
	}

	_ttest center "Ho: mean(@`xname'@) = @`m0'@"

/* Display Ha. */

	local tt : di %8.4f `t'
	local p1 : di %8.4f `pl'
        local p2 : di %8.4f `p'
        local p3 : di %8.4f `pr'

        di
        _ttest center "Ha: mean < @`m0'@"  /*
	*/            "Ha: mean ~= @`m0'@" /*
	*/            "Ha: mean > @`m0'@"

	_ttest center "    t = @`tt'@"    /*
	*/            "      t = @`tt'@"  /*
	*/            "    t = @`tt'@"
	_ttest center "P < t = @`p1'@"   /*
	*/            "P > |t| = @`p2'@" /*
	*/            "P > t = @`p3'@"

	global S_1  `n'
	global S_2  `m'
	global S_5 =`n' - 1
	global S_6  `t'
	global S_7  `p'
	global S_8  `pl'
	global S_9  `pr'
	global S_10 `se'
	global S_11 `s'
end

program define TwoTest
        local n1      "`1'"
        local m1      "`2'"
        local s1      "`3'"
        local n2      "`4'"
        local m2      "`5'"
        local s2      "`6'"
        local level   "`7'"
        local xname   "`8'"
        local yname   "`9'"
	local unequal "`10'"
	local welch   "`11'"

/* Compute statistics. */

        if `n1' == 1 { local s1 . } /* error check allows std dev to be 0 */
        if `n2' == 1 { local s2 . }

	tempname var

	if "`unequal'"!="" | "`welch'"!="" {
		local un "un"

		scalar `var' = (`s1')^2/`n1' + (`s2')^2/`n2'

		if "`welch'"!="" {
			local df = -2 + `var'^2            /*
			*/ /( ((`s1')^2/`n1')^2/(`n1'+1)   /*
			*/  + ((`s2')^2/`n2')^2/(`n2'+1) )
		}
		else {
			local df = `var'^2                 /*
			*/ /( ((`s1')^2/`n1')^2/(`n1'-1)   /*
			*/  + ((`s2')^2/`n2')^2/(`n2'-1) )
		}
	}
	else {
		local df = `n1' + `n2' - 2
		scalar `var' = (((`n1'-1)*(`s1')^2+(`n2'-1)*(`s2')^2)/`df') /*
		*/             *(1/`n1' + 1/`n2')
	}

	local t  = (`m1'-`m2')/sqrt(`var')
	local se = sqrt(`var')

	local p = tprob(`df',`t')
	if `t' < 0 {
		local pl = `p'/2
		local pr = 1 - `pl'
	}
	else {
		local pr = `p'/2
		local pl = 1 - `pr'
	}

/* Display table of mean, std err, etc. */

	di _n in gr "Two-sample t test with `un'equal variances"

	_ttest header `level' "`xname'"

	if "`xname'"=="" { local xname "x" }
	if "`yname'"=="" { local yname "y" }

	_ttest table `level' "`xname'" `n1' `m1' `s1'
	_ttest table `level' "`yname'" `n2' `m2' `s2'
	di in gr "---------+" _dup(68) "-"

/* Display combined mean, etc. */

	local n = `n1' + `n2'

	tempname m s /* use scalars to get every possible digit of accuracy */

	scalar `m' = (`n1'*`m1'+`n2'*`m2')/`n'
	scalar `s' = sqrt(( (`n1'-1)*(`s1')^2 + `n1'*(`m'-`m1')^2 /*
	*/                + (`n2'-1)*(`s2')^2 + `n2'*(`m'-`m2')^2)/(`n'-1))

	_ttest table `level' "combined" `n' `m' `s'
	di in gr "---------+" _dup(68) "-"

/* Display difference. */

	_ttest dtable `level' "diff" `n' `m1'-`m2' `se' `df'

	di in gr _dup(78) "-"

	if "`welch'"!="" {
		di in gr "Welch's degrees of freedom: " in ye %8.0g `df' _n
	}
	else if "`unequal'"!="" {
		di in gr "Satterthwaite's degrees of freedom: " /*
		*/ in ye %8.0g `df' _n
	}
	else	di in gr "Degrees of freedom: " in ye `df' _n

/* Display error messages. */


/* Display Ho. */

	_ttest center "Ho: mean(@`xname'@) - mean(@`yname'@) = diff = 0"

/* Display Ha. */

	local tt : di %8.4f `t'
	local p1 : di %8.4f `pl'
        local p2 : di %8.4f `p'
        local p3 : di %8.4f `pr'

        di
        _ttest center "Ha: diff < 0" "Ha: diff ~= 0" "Ha: diff > 0"

	_ttest center "    t = @`tt'@"    /*
	*/            "      t = @`tt'@"  /*
	*/            "    t = @`tt'@"
	_ttest center "P < t = @`p1'@"   /*
	*/            "P > |t| = @`p2'@" /*
	*/            "P > t = @`p3'@"

	global S_1  `n1'
   	global S_2  `m1'
	global S_3  `n2'
	global S_4  `m2'
	global S_5  `df'
	global S_6  `t'
	global S_7  `p'
	global S_8  `pl'
	global S_9  `pr'
	global S_10 `se'
	global S_11 `s1'
	global S_12 `s2'
	global S_13 = `s'
end
