*! version 5.0.7  24nov1997
program define weibull
	version 5.0

	if "`1'"=="" | substr("`1'",1,1)=="," {
		if "$S_E_cmd" != "weibull" { error 301 }
		if "$S_E_frm2"=="hazard" {
			local options "HR"
		}
		else	local options "TR"
		local options "`options' noCOEF noHEAD Level(integer $S_level)"
		parse "`*'"
	}
	else {
		local varlist "req ex min(1)"
		local if "opt"
		local in "opt"
		local weight "fweight aweight iweight pweight noprefix"
		local options "CLuster(string) noCOEF HAzard noHEAD Level(integer $S_level) noLOG Dead(string) HR T0(string) TR Robust SCore(string) *"
		parse "`*'"
		parse "`varlist'", parse(" ")
		local t "`1'"
		mac shift
		local rhs "`*'"

		if "`hazard'"!="" {
			if "`tr'"!="" {
				di in red "tr invalid with hazard option"
				exit 198
			}
		}
		else if "`hr'"!="" {
			local hazard "hazard"
		}

		if "`t0'"!="" {
			unabbrev "`t0'", max(1) 
			local t0 "$S_1"
		}

		if "`weight'"!="" {
			tempvar wv
			qui gen double `wv' = `exp'
			local w "[`weight'=`wv']"
			if "`weight'"=="pweight" {
				local robust "robust"
			}
		}
		if "`score'"!="" {
			local n : word count `score'
			if `n'!=2 { 
				di in red "score() invalid"
				exit 198
			}
			confirm new var `score'
			parse "`score'", parse(" ")
			local score1 "`1'"
			local score2 "`2'"
		}
		if "`cluster'"!="" {
			unabbrev `cluster', max(1)
			local cluster "$S_1"
			local robust "robust"
		}
		if "`robust'"!="" & "`score'"=="" {
			tempvar score1 score2
		}

		if "`robust'"!="" & "`weight'"=="iweight" { 
			di in red "may not specify robust with iweights"
			exit 101
		}


		tempvar touse
		mark `touse' `w' `if' `in'
		markout `touse' `t' `rhs' `dead' `t0'
		markout `touse' `cluster', strok

		if "`dead'"!="" {
			unabbrev `dead', max(1)
			local dead "$S_1"
			capture assert `dead'==0 | `dead'==1 if `touse'
			if _rc {
				tempvar mydead
				qui gen byte `mydead'=`dead'!=0 if `touse'
				local dead "`mydead'"
			}
		}
		else {
			tempvar dead
			qui gen byte `dead'=1
		}

		if "`t0'"=="" {
			local t0 0
		}
		capture assert `t0' < `t' if `touse'
		if _rc {
			di in red "`t0' >= `t' in some obs."
			exit 498
		}

		_rmcoll `rhs' `w' if `touse'
		local rhs "$S_1"
		global S_1

		qui count if `touse' 
		local nobs = _result(1)		/* may be reset subsequently */
		if `nobs'<2 {
			if _result(1)==0 { error 2000 } 
			error 2001
		}

		if "`weight'" != "" {
			qui replace `wv'=0 if !`touse'
			if "`weight'"=="aweight" | "`weight'"=="pweight" {
				qui summ `wv' if `touse', meanonly
				qui replace `wv' = `wv'/_result(3) if `touse'
			}
			else if "`weight'"=="fweight" {
				qui summ `wv', meanonly
				local nobs = round(_result(18),1)
			}
			global EREGw "`wv'"
		}
		else	global EREGw "`touse'"

		global EREGd "`dead'"
		global EREGt "`t'"
		global EREGt0 "`t0'"

		tempvar mysamp num den
		tempname b f V  g b0

		quietly {
			replace $EREGw = 0 if $EREGw==.
			gen double `num' = sum($EREGw*$EREGd)
			gen double `den' = sum($EREGw*($EREGt-$EREGt0))
			local cons = -ln(`num'[_N]/`den'[_N])
			drop `num' `den'
			gen double `num' = sum($EREGw*ln($EREGd*$EREGt))
			global EREGa = `num'[_N]
			drop `num'

			mat `b0' = ( `cons', 0 )
			mat colnames `b0' = $EREGt:_cons ln_p:_cons
		}

		ml begin
		ml function weib_lf
		ml method deriv2
		eq `t' : `t'
		eq ln_p :
		ml model `b' = `t' ln_p, depv(10) from(`b0')
		eq drop `t' ln_p
		ml sample `mysamp', noauto
		qui ml maximize `f' `V'

		mat `b0' = `b'
		local f0 = `f'
		drop `mysamp'

		ml begin 
		ml function weib_lf
		ml method deriv2 
		eq `t' : `t' `rhs'
		eq ln_p : 
		ml model `b' = `t' ln_p,  depv(10) from(`b0')
		eq drop `t' ln_p
		ml sample `mysamp', noauto	/* noauto for speed */
		if "`log'"=="" {
			ml maximize `f' `V', `options'
		}
		else	qui ml maximize `f' `V', `options'

		if "`hazard'"!="" {
			Remetric `b' `V'
			local ttl "log relative hazard form"
			if "`score1'"!="" {
				ScoHaz `b' -> `score1' `score2'
			}
		}
		else {
			local ttl "log expected time form"
			if "`score1'"!="" {
				ScoTim `b' -> `score1' `score2'
			}
		}

		ml post weibull, lf0(`f0') /* pr2 */ obs(`nobs') title("`ttl'")
		global S_E_cmd
		global S_E_t0 "`t0'"

		if "`robust'"!="" {
			if "`cluster'"!="" {
				local cluster "cluster(`cluster')"
			}
			_robust `score1' `score2' `w' if `touse', `cluster' nofatal
			capture test `rhs', min
			global S_E_chi2 = _result(6)
			global S_E_mdf = _result(3)
		}
		if "`hazard'"=="" {
			global S_E_frm2 "time"
		}
		else	global S_E_frm2 "hazard"
		global S_E_cmd "weibull"

		global EREGw
		global EREGd
		global EREGt
		global EREGt0
		global EREGa
	}

	if "`coef'"=="" {
		if "`hr'" != "" {
			local hr "eform(Haz. Ratio)"
		}
		else if "`tr'"!="" {
			local hr "eform(Tm. Ratio)"
		}
		if "`head'"=="" {
			di _n in gr /*
			*/ "Weibull regression -- entry time $S_E_t0" _c
			ml mlout weibull, `hr' level(`level') first
		}
		else	mat mlout, `hr' level(`level') first
		local lnp = [ln_p]_b[_cons]
		local se = [ln_p]_se[_cons]
		local iz = invnorm(1-(1-`level'/100)/2)
		local lnp0 = `lnp'-`iz'*`se'
		local lnp1 = `lnp'+`iz'*`se'
		local z = [ln_p]_b[_cons]/[ln_p]_se[_cons]
		di in gr "    ln p |  " in ye %9.0g (`lnp') /*
		*/ "  " %9.0g (`se') " " %10.3f (`z') /* 
		*/ " " %7.3f (normprob(-abs(`z'))*2) /*
		*/ _col(58) %9.0g (`lnp0') "   " %9.0g (`lnp1')
		di in gr "       p |  " in ye %9.0g exp(`lnp') /*
		*/ _col(58) %9.0g exp(`lnp0') "   " %9.0g exp(`lnp1')
		di in gr "     1/p |  " in ye %9.0g 1/exp(`lnp') /*
		*/ _col(58) %9.0g 1/exp(`lnp1') "   " %9.0g 1/exp(`lnp0')
		di in gr _dup(78) "-"
	}
end

program define Remetric /* b V */
	local baug "`1'"
	local D "`2'"

	tempvar I lnF1 lnF0 w
	tempname b ln_p s p

	Getvals `baug' -> `b' `ln_p' `p' `s'

	local rhs : colnames(`b')

	/* note, we use p = exp(s), s=0 means p=1; s=ln(p) */

	quietly {
		mat score double `I' = `b' if $EREGw
		replace `I' = -`I'*`p'
		gen double `lnF1' = -exp(`I')*($EREGt)^`p' if $EREGw
		gen double `lnF0' = -exp(`I')*($EREGt0)^`p' if $EREGw


		tempname dbds dsds g2

		parse "`rhs'", parse(" ")
		local n : word count `rhs'
		if "``n''"=="_cons" {
			local `n'
		}

		local lnt1 "ln($EREGt)"
		if "$EREGt0"!="0" {
			tempvar lnt0
			gen double `lnt0'=cond($EREGt0==0,0,ln($EREGt0))
		}
		else	local lnt0 0


					/* 2nd derivatives 		*/
					/* D = d2lnL / dbdb 		*/
		if "`*'"=="" {
			mat accum `D' = $EREGt [iw=$EREGw*(`lnF1'-`lnF0')]
			mat `D' = `D'[2,2]
		}
		else	mat accum `D' = `*' [iw=$EREGw*(`lnF1'-`lnF0')]

					/* dbds = d2lnL / dbds		*/
		gen double `w' = $EREGw*`p'*(`lnF1'*`lnt1' - `lnF0'*`lnt0')
		mat vecaccum `dbds' = `w' `*'
		drop `w'


		gen double `w' = sum($EREGw*( /*
		*/	`lnF1'*`p'*`lnt1'*(1+`p'*`lnt1') - /*
		*/	`lnF0'*`p'*`lnt0'*(1+`p'*`lnt0') + /*
		*/	$EREGd*(`p'*`lnt1') /*
		*/ ))
		mat `dsds' = J(1,1,0)
		mat `dsds'[1,1] = `w'[_N]
		drop `w'

		mat colnames `D' = $EREGt:
		mat rownames `D' = $EREGt:
		mat colnames `dbds' = $EREGt:
		mat rownames `dbds' = ln_p:_cons
		mat `D' = `D' \ `dbds'
		mat `dbds' = `dbds' '
		mat `dbds' = `dbds' \ `dsds' 
		mat `D' = `D' , `dbds'
		mat `D' = -1 * `D'
		mat `D' = syminv(`D')

		scalar `p' = -`p'
		mat `b' = `p' * `b'
		mat `baug' = `b', `ln_p'
	}
end

program define Getvals /* baug -> b ln_p p s */
	local baug "`1'"
	local b "`3'"
	local ln_p "`4'"
	local p "`5'"
	local s "`6'"

	mat `b' = `baug'[1,"$EREGt:"]
	mat `ln_p' = `baug'[1,"ln_p:"]
	scalar `s' = `ln_p'[1,1]
	scalar `p' = exp(`s')
end

program define ScoHaz /* b -> sc1 sc2 */
	local baug "`1'"
	local sc1 "`3'"
	local sc2 "`4'"

	tempname b ln_p p s
	Getvals `baug' -> `b' `ln_p' `p' `s'

	tempvar I lnF0 lnF1 lnt0
	quietly {
		gen double `sc1' = . 
		gen double `sc2' = . 
		mat score double `I' = `b' if $EREGw
		gen double `lnF1'= -exp(`I')*($EREGt)^`p'
		gen double `lnF0'= -exp(`I')*($EREGt0)^`p'

		replace `sc1' = `lnF1'-`lnF0'+$EREGd if $EREGw

		if "$EREGt0"!="0" {
			gen double `lnt0' = cond($EREGt0==0,0,ln($EREGt0))
		}
		else	local lnt0 0
		local lnt1 "ln($EREGt)"
		replace `sc2' = `lnF1'*`p'*`lnt1' - /*
		*/ 		`lnF0'*`p'*`lnt0' + /* 
		*/		$EREGd*(1+`p'*`lnt1') if $EREGw
	}
end

program define ScoTim /* b -> sc1 sc2 */
	local baug "`1'"
	local sc1 "`3'"
	local sc2 "`4'"

	tempname b ln_p p s
	Getvals `baug' -> `b' `ln_p' `p' `s'

	tempvar I lnF0 lnF1 lnt0
	quietly {
		gen double `sc1' = . 
		gen double `sc2' = . 
		mat score double `I' = `b' if $EREGw

		gen double `lnF1'= -exp(-`I'*`p')*($EREGt)^`p'
		gen double `lnF0'= -exp(-`I'*`p')*($EREGt0)^`p'

		replace `sc1' = (`lnF0'-`lnF1')*`p' - $EREGd*`p' if $EREGw

		if "$EREGt0"!="0" {
			gen double `lnt0' = cond($EREGt0==0,0,ln($EREGt0))
		}
		else	local lnt0 0
		local lnt1 "ln($EREGt)"

		replace `sc2' = `lnF1'*(`lnt1'-`I')*`p' - /*
		*/		`lnF0'*(`lnt0'-`I')*`p' + /*
		*/		$EREGd*(1+`p'*`lnt1'-`I'*`p') /*
		*/ if $EREGw
	}
end
exit
